package org.fuseri.modulelanguageschool.lecture;

import com.fasterxml.jackson.databind.ObjectMapper;
import org.fuseri.model.dto.lecture.LectureDto;
import org.fuseri.model.dto.user.UserDto;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.http.MediaType;
import org.springframework.test.web.servlet.MockMvc;

import java.time.LocalDateTime;
import java.time.Month;

import static org.hamcrest.MatcherAssert.assertThat;
import static org.hamcrest.Matchers.instanceOf;
import static org.hamcrest.Matchers.is;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.patch;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.jsonPath;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

@SpringBootTest
@AutoConfigureMockMvc
public class LectureTest {
    @Autowired
    ObjectMapper objectMapper;
    @Autowired
    private MockMvc mockMvc;

    public static String asJsonString(final Object obj) {
        try {
            return new ObjectMapper().writeValueAsString(obj);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    @Test
    void testSettingLecturer() throws Exception {
        LectureDto expectedResponse = new LectureDto(
                LocalDateTime.of(2045, Month.JUNE, 30, 12, 0, 0),
                LocalDateTime.of(2045, Month.JUNE, 30, 14, 0, 0),
                "Learning how to spell deprecated",
                10, "0", "0");
        String response = mockMvc.perform(patch("/lectures/setLecturer/0")
                        .content(asJsonString(new UserDto()))
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.lecturerId").value("0"))
                .andExpect(jsonPath("$.courseId").value("0"))
                .andExpect(jsonPath("$.topic").value("Learning how to spell deprecated"))
                .andExpect(jsonPath("$.capacity").value(10))
                .andReturn().getResponse().getContentAsString();
        LectureController lectureController = objectMapper.readValue(response, LectureController.class);
        assertThat("response", lectureController.setLecturer("0", new UserDto()), is(instanceOf(expectedResponse.getClass())));
    }

}
